import pandas as pd
import regex as re
from bs4 import BeautifulSoup as soup

def getCodeFromSubSection(subSection):
    if subSection.startswith("https://xbrl.us/data-rule/dqc"):
        return 60000000
    code = sum(int(n) * 10**(i*2)
                for i,n in enumerate(reversed(subSection.split(".")), start=1))
    return code

def generateDFWithNumberColumn(filename):
    df = readHtml(filename)
    df.insert(1, "Number", df["Subsection"].apply(getCodeFromSubSection))
    return df

def readHtml(filename):
    tables = pd.read_html(f"../{filename}")
    df = tables[0]
    return df

def sanitized_string(string):
    if '<' in string and '>' in string:
        # we may have valid html
        # using this to detect valid HTML tags in elements that support HTML.
        valid_tags = ["a","abbr","address","area","article","aside","audio","b","base","bdi","bdo","blockquote","body","br","button","canvas","caption","cite","code","col","colgroup","data","datalist","dd","del","details","dfn","dialog","div","dl","dt","em","embed","fieldset","figcaption","figure","footer","form","h1","h2","h3","h4","h5","h6","head","header","hgroup","hr","html","i","iframe","img","input","ins","kbd","label","legend","li","link","main","map","mark","math","menu","menuitem","meta","meter","nav","noscript","object","ol","optgroup","option","output","p","param","picture","pre","progress","q","rb","rp","rt","rtc","ruby","s","samp","script","search","section","select","slot","small","source","span","strong","style","sub","summary","sup","svg","tbody","td","template","textarea","tfoot","th","thead","time","title","tr","track","u","ul","var","video","wbr"]
        tag_endings_string = ''
        tag_beginnings_string = ''
        for i, tag in enumerate(valid_tags):
            if i > 0:
                tag_endings_string += '|'
                tag_beginnings_string += '|'
            tag_endings_string += f'{tag}>|'
            tag_endings_string += f'/{tag}>'
            tag_beginnings_string += f'<{tag}|'
            tag_beginnings_string += f'</{tag}'

        match_lt = '<(?!(' + tag_endings_string + '))'
        match_gt = '(?<!(' + tag_beginnings_string + '))>'

        regex_lt = re.compile(match_lt)
        regex_gt = re.compile(match_gt)

        string = string.replace('&', '&amp;') # & must be replaced first
        string = regex_lt.sub('&lt;', string)
        string = regex_gt.sub('&gt;', string)
        return string
    return string

def generateHTMLfromDataFrame(df, tp="Warnings"):
    top_section = f"""\
<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<html lang="ENG">
<!--Do not edit.-->
<head>
    <title>EDGAR XBRL Validation {tp}</title>
    <meta charset="UTF-8"/>
    <style type="text/css">
        table{{
            width: 100%;
            font-family: Verdana, Arial, Helvetica;
            font-size: 9pt;
        }}

        table,
        th,
        td{{
            border: 1px solid black;
            border-collapse: collapse;
        }}

        tr{{
            text-align: left;
            valign: middle;
        }}

        .warnitem th{{
            text-align: center;
            font-weight: bold;
        }}

        .warnitem td:nth-child(1){{
            width: 25%;
            vertical-align: middle;
        }}
        .warnitem td:nth-child(2){{
            width: 15%;
            text-align: center;
            vertical-align: middle;
        }}
        .warnitem td:nth-child(3){{
            width: 45%;
        }}
        .warnitem td:nth-child(4){{
            width: 15%;
            text-align: center;
            vertical-align: middle;
        }}

        .modified{{
            color: white;
        }}</style>
</head>
<body>
    <h2>EDGAR XBRL Validation {tp}</h2>
    <table class="warnitem">
    <thead>
        <tr>
            <th>Message ID</th>
            <th>Number</th>
            <th>Message</th>
            <th>Subsection</th>
        </tr>
    </thead>
    <tbody>"""
    bottom_section = """
    </tbody>
    </table>
</body>
</html>"""
    out = top_section
    for _, row in df.iterrows():
        out += f"""
        <tr>
            <td>{row[0]}</td>
            <td>{row[1]}</td>
            <td>{sanitized_string(row[2])}</td>
            <td>{row[3]}</td>
        </tr>"""
    out += bottom_section
    return out

if __name__ == "__main__":
    files = [("xbrlerrors.htm", "filing-fee-exhibit-xbrlerrors.htm", "Errors", "../xbrlerrors.htm"),
             ("xbrlwarnings.htm", "filing-fee-exhibit-xbrlwarnings.htm", "Warnings", "../xbrlwarnings.htm"),
             ]
    for efmFile, FTFile, tp, outFile in files:
        df = generateDFWithNumberColumn(efmFile)
        dfFT = readHtml(FTFile)
        df = pd.concat([df, dfFT])
        df = df.sort_values("Message ID")
        html = generateHTMLfromDataFrame(df, tp=tp)
        with open(outFile, "w") as f:
            f.write(html)